// The MIT License

// Copyright (c) 2010 Massachusetts Institute of Technology

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:

// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Author: Richard Uhler ruhler@mit.edu

import ClientServer::*;
import FixedPoint::*;
import GetPut::*;
import StmtFSM::*;


import SquareRoot::*;

module test (Empty);

    SquareRoot#(Bit#(32)) sqrt <- mkSquareRoot(2);
    SquareRoot#(FixedPoint#(16,16)) fpsqrt <- mkFixedPointSquareRoot(2);
    Reg#(Bool) failed <- mkReg(False);

    // Generate a single test case.
    // Warning: These are not parallel composable.
    // Input: x, expected q, expected r
    function Stmt atest(x, qe, re);
        return (seq
            sqrt.request.put(x);
            action
                match {.qa, .ra} <- sqrt.response.get();
                if (qa != qe || ra != re) begin
                    $display("FAIL: sqrt %d = %d r %d, but got %d r %d", x, qe, re, qa, ra);
                    failed <= True;
                end
            endaction
        endseq);
    endfunction

    // Generate a test case for the fixed point square root.
    // The remainder part is ignored.
    function Stmt fptest(FixedPoint#(16, 16) x, FixedPoint#(16, 16) qe);
        return (seq
            fpsqrt.request.put(x);
            action
                match {.qa, .ra} <- fpsqrt.response.get();
                if (qa != qe) begin
                    $write("FAIL: sqrt(");
                    fxptWrite(10, x);
                    $write(") = ");
                    fxptWrite(10, qe);
                    $write (" but got ");
                    fxptWrite(10, qa);
                    $display("");
                    failed <= True;
                end
            endaction
        endseq);
    endfunction

    Stmt dotest = (
        seq
            atest(16, 4, 0);
            atest(18, 4, 2);
            atest(25, 5, 0);
            atest(1, 1, 0);
            atest(13511, 116, 55);
            atest(0, 0, 0);
            atest(1638400, 1280, 0);

            fptest(25.0, 5.0);
            fptest(17.0, 4.12310562);
            fptest(0.1421, 0.376961536);

            $display(failed ? "FAILED" : "PASSED");
        endseq
    );
    
    mkAutoFSM(dotest);
endmodule


